/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: CliStarter.java 20828 2011-02-03 10:12:20Z benoitf $
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.autostart.clistarter;

import java.io.File;
import java.security.ProtectionDomain;
import java.util.Scanner;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.jonas.autostart.starter.Starter;
import org.ow2.jonas.autostart.starter.StarterException;
import org.ow2.jonas.autostart.utility.Utility;

/**
 * The JOnAS Starter command line client.
 * @author skabore
 */
public class CliStarter {

    /**
     * The length of arguments.
     */
    private static final int THREE_ARGS = 3;

    /**
     * The logger for traces.
     */
    private static final Logger logger = Logger.getLogger(CliStarter.class.getName());

    /**
     * Build CliStarter.
     * @param domaine the domaine name
     * @param args the command line arguments
     */
    public CliStarter(final ProtectionDomain domaine, final String[] args) {
        Starter starter = new Starter();
        File javaHome = checkJavaHome();
        starter.setJavaHome(javaHome);
        starter.setVerboseMode(false);
        int paramLength = args.length;
        switch (paramLength) {
        case 0:
            logger.info("JOnAS Root will be installed in " + Utility.getDefaultLocation()
                    + System.getProperty("file.separator") + starter.getJonasRootFolder());
            logger.info("Press Enter to continue or any key + Enter" + " to abort.");
            Scanner sc = new Scanner(System.in);
            String entree = sc.nextLine();
            if (entree.length() == 0) {
                starter.setJonasRootInstallLocation(new File(Utility.getDefaultLocation()));
            } else {
                System.exit(0);
            }
            break;

        case 2:
            if (args[0].equals("-r") || args[0].equals("--jonas-root")) {
                checkDirectory(new File(args[1]));
                File jonasRootPath = new File(args[1]);
                starter.setJonasRootInstallLocation(jonasRootPath);
            } else if (args[0].equals("-s") || args[0].equals("-stop") || args[0].equals("--stop-jonas")) {
                File jonasRootPath = new File(args[1]);
                if (Utility.isValidJOnASRootFolder(jonasRootPath)) {
                    starter.setJonasRoot(jonasRootPath);
                    starter.stopJonas();
                    System.exit(1);
                } else {
                    logger.log(Level.INFO, "JOnAS Root folder is" + " not valid !");
                    logger.log(Level.INFO, "Please enter a valid folder.");
                    System.exit(1);
                }
            } else {
                printUsage();
                System.exit(1);
            }
            break;

        case 3:
            // Run an existing JOnAS server
            if (args[0].equals("-e") || args[0].equals("--existing") || args[0].equals("--existing-jonas")) {
                starter.setUseEmbeddedJonas(false);
                File jonasRootPath = new File(args[2]);
                if (Utility.isValidJOnASRootFolder(jonasRootPath)) {
                    starter.setJonasRoot(jonasRootPath);
                } else {
                    logger.log(Level.INFO, "JOnAS Root folder is" + " not valid !");
                    logger.log(Level.INFO, "Please enter a valid folder.");
                    System.exit(1);
                }
            } else {
                printUsage();
                System.exit(1);
            }
            break;

        // Create new JOnAS Base ??? When 'reconf' option is specified
        case 4:
            if (args[0].equals("-r") || args[0].equals("--jonas-root")) {
                checkDirectory(new File(args[1]));
                File jonasRootPath = new File(args[1]);
                starter.setJonasRootInstallLocation(jonasRootPath);
                if (args[2].equals("-b") || args[2].equals("--jonas-base")) {
                    checkDirectory(new File(args[THREE_ARGS]));
                    File jonasBaseLocation = new File(args[THREE_ARGS]);
                    starter.setJonasBaseInstallLocation(jonasBaseLocation);
                    if (jonasBaseLocation.isDirectory() && !starter.jonasBaseIsSet()) {
                        // starter.createJonasBaseTemplate();
                    } else {
                        starter.setJonasBase(null);
                    }
                } else {
                    printUsage();
                    System.exit(1);
                }
            } else {
                printUsage();
                System.exit(1);
            }
            break;

        case 5:
            if (args[0].equals("-e") || args[0].equals("--existing") || args[0].equals("--existing-jonas")) {
                if (args[1].equals("-r") || args[1].equals("--jonas-root")) {
                    File jonasRootPath = new File(args[2]);
                    if (Utility.isValidJOnASRootFolder(jonasRootPath)) {
                        starter.setUseEmbeddedJonas(false);
                        starter.setJonasRoot(jonasRootPath);
                        if (args[3].equals("-b") || args[2].equals("--jonas-base")) {
                            File jonasBasePath = new File(args[4]);
                            if (Utility.isValidJOnASBaseFolder(jonasBasePath)) {
                                starter.setJonasBase(jonasBasePath);
                            } else {
                                logger.log(Level.INFO, "JOnAS Base folder is" + " not valid !");
                                starter.setJonasBase(null);
                            }
                        }
                    } else {
                        logger.log(Level.INFO, "JOnAS Root folder is" + " not valid !");
                        logger.log(Level.INFO, "Please enter a valid folder.");
                        System.exit(1);
                    }
                } else {
                    printUsage();
                    System.exit(1);
                }
            } else {
                printUsage();
                System.exit(1);
            }
            break;

        default:
            printUsage();
            System.exit(1);
        }
        String path = domaine.getCodeSource().getLocation().getFile();
        if (Utility.getPlatform().equalsIgnoreCase("windows")) {
            path = path.replace("/", "\\");
            path = path.replace("%20", " ");
            path = path.substring(1);
        }

        if (starter.getUseEmbeddedJonas()) {
            starter.createTmpWorkDirectory();
            starter.setStarterLocation(new File(path));

            /**
             * Extracts JOnAS.
             */
            logger.log(Level.INFO, " *************** Extracting JOnAS... *************** ");
            logger.log(Level.INFO, "Please wait !");
            try {
                starter.extractJonas();
            } catch (StarterException e) {
                logger.log(Level.SEVERE, "error", e);
            }
            Utility.deleteDirectory(starter.getWorkdirectory());
            starter.setUseEmbeddedJonas(false);
        }

        /**
         * Starts JOnAS server.
         */
        logger.log(Level.INFO, " *************** Starting JOnAS server... *************** ");
        starter.startJonas();
        if (starter.jonasIsRunning()) {
            logger.log(Level.INFO, "JOnAS server started at localhost:" + starter.getJonasDefaultPort() + ".");
        } else {
            logger.log(Level.INFO, "Failed to start JOnAS server at localhost:" + starter.getJonasDefaultPort() + ".");
        }
    }

    /**
     * Checks if JAVA_HOME value is set.
     * @return JAVA_HOME value
     */
    private File checkJavaHome() {
        String javaHome = System.getenv("JAVA_HOME");
        if (javaHome == null) {
            logger.info("JAVA_HOME is not set.");
            logger.info("Enter JAVA_HOME value : ");
            Scanner sc = new Scanner(System.in);
            String javaHomeFolder = sc.nextLine();
            if (javaHomeFolder.length() == 0) {
                System.exit(0);
            } else {
                javaHome = javaHomeFolder;
            }
        }
        return new File(javaHome);
    }

    /**
     * Print the usage.
     */
    private void printUsage() {
        logger.info("Error : invalid command or option.");
        logger.info("Usage : ");
        logger.info("java -jar jonas-starter.jar");
        logger.info("java -jar jonas-starter.jar -gui");
        logger.info("java -jar jonas-starter.jar -reconf");
        logger.info("java -jar jonas-starter.jar -r|--jonas-root" + " <JOnAS Root install location> "
                + "[-b|--jonas-base <JOnAS Base install location>]");
        logger.info("java -jar jonas-starter.jar -e|--existing-jonas" + " -r|--jonas-root <JOnAS Root location> "
                + "[-b|--jonas-base <JOnAS Base location>]");
        logger.info(" [-s|-stop|--stop-jonas <JOnAS Root location>]");
    }

    /**
     * Checks install location exists.
     * @param path the location path.
     */
    private void checkDirectory(final File path) {
        if (!path.exists() || path.isDirectory()) {
            logger.log(Level.INFO, path.getAbsolutePath() + " doesn't exist !");
            logger.log(Level.INFO, "Create it ? (Yes/No) ");
            Scanner sc = new Scanner(System.in);
            String entry = sc.nextLine();
            if (entry.length() == 0) {
                entry = "yes";
            }
            if (entry.toLowerCase().contains("y")) {
                Utility.createDirectory(path);
            } else {
                System.exit(0);
            }
        }
    }
}
